function getHighlighterBar() {
  return $('#highlighter');
}

function renderEndDateField() {
  var duration = $.trim($("#duration").val());
  var durationUnits = $.trim($("#hourSelect").val());
  var startDate = new Date( $.trim($("#start_date").val()) );
  var endDate = dateAdd(startDate, durationUnits, duration);
  $("#end_date").val( formatDate(endDate) );
}

function renderDurationField() {
  var eventDurationHours = getEventWindowHours() * getHighlighterBar().width() / getEventWindowWidth();

  var duration;
  var durationUnits = $.trim($("#hourSelect").val());

  switch( durationUnits ) {
    case 'day':
      duration = (eventDurationHours / 24).toFixed(2);
      break;
    case 'hour':
      duration = (eventDurationHours).toFixed(2);
      break;
    case 'minute':
      duration = parseInt(eventDurationHours * 60);
      break;
    default:
      duration = parseInt(eventDurationHours * 60);
  }

  $("#duration").val(duration);
  $("#resource_planner_event_duration").val(duration);
}

function onHighlighterBarUpdate(event, ui, obj) {
  var isRTL = $('#fullcalendar_scheduler').fullCalendar('option', 'isRTL');
  var view = $('#fullcalendar_scheduler').fullCalendar('getView');
  var viewStartDate = new Date(view.intervalStart.toDate());
  var viewEndDate = new Date(view.intervalEnd.toDate());

  var viewStartTime = viewStartDate.getTime();
  var viewEndTime = viewEndDate.getTime();
  var viewDuration = viewEndTime - viewStartTime;

  var eventStartTimeHours;
  var eventEndTimeHours;
  var eventDurationHours = getEventWindowHours() * obj.width() / getEventWindowWidth();

  if( isRTL ) {
    eventEndTimeHours   = getEventWindowHours() - (getEventWindowHours() * ui.position.left / getEventWindowWidth());
    eventStartTimeHours = eventEndTimeHours - eventDurationHours;
  } else {
    eventStartTimeHours = getEventWindowHours() * ui.position.left / getEventWindowWidth();
    eventEndTimeHours   = eventStartTimeHours + eventDurationHours;
  }

  // Date object methods
  // setDate()	Sets the day of the month of a date object
  // setFullYear()	Sets the year of a date object
  // setHours()	Sets the hour of a date object
  // setMilliseconds()	Sets the milliseconds of a date object
  // setMinutes()	Set the minutes of a date object
  // setMonth()	Sets the month of a date object

  var startDate = dateAdd(viewStartDate, 'day', parseInt(eventStartTimeHours/24));
  startDate.setHours( Math.floor(eventStartTimeHours % 24) );
  startDate.setMinutes( 60 * (eventStartTimeHours % 1).toFixed(10) );

  // var endDate = dateAdd(viewStartDate, 'day', parseInt(eventEndTimeHours/24));
  // endDate.setHours( Math.floor(eventEndTimeHours % 24) );
  // endDate.setMinutes( 60 * (eventEndTimeHours % 1).toFixed(10) );

  $("#start_date").val( formatDate(startDate) );
  // $("#end_date").val( formatDate(endDate) );

  renderDurationField();
  renderEndDateField();
}

function renderHighlighterBar(trigger) {
  if( getHighlighterBar().length == 0 ) {
    $('#fullcalendar_scheduler tbody.fc-body td.fc-time-area div.fc-scroller div.fc-scroller-canvas').prepend($('<div id="highlighter" draggable="true"></div>'));
  }

  getHighlighterBar().on('drag', function( event, ui ) {
    onHighlighterBarUpdate(event, ui, $(this));
  });

  getHighlighterBar().on('resize', function( event, ui ) {
    onHighlighterBarUpdate(event, ui, $(this));
  });

  var start_date = $.trim($("#start_date").val());
  var end_date = $.trim($("#end_date").val());
  var duration = $.trim($("#duration").val());
  var durationUnits = $.trim($("#hourSelect").val());

  var isRTL = $('#fullcalendar_scheduler').fullCalendar('option', 'isRTL');
  var view = $('#fullcalendar_scheduler').fullCalendar('getView');

  var viewStartDate = new Date(view.intervalStart.toDate());
  var viewEndDate = new Date(view.intervalEnd.toDate());

  var viewStartTime = viewStartDate.getTime();
  var viewEndTime = viewEndDate.getTime();
  var viewDuration = viewEndTime - viewStartTime;

  eventWindowH = $('#fullcalendar_scheduler tbody.fc-body td.fc-time-area div.fc-scroller');
  eventWindowW = $('#fullcalendar_scheduler tbody.fc-body td.fc-time-area div.fc-scroller-canvas');

  getHighlighterBar().height( getEventWindowHeight() );
  getHighlighterBar().draggable({ axis: "x" });
  getHighlighterBar().resizable({ handles: "e, w" });

  if( start_date.length && (end_date.length || duration.length) ) {
    var startDate = new Date(start_date);

    switch( trigger ) {
      case 'date':
        if(end_date.length) {
          var endDate = new Date(end_date);

          if( isRTL ) {
            getHighlighterBar().css('left',  (endDate.getTime()/viewDuration) * getEventWindowWidth() );
          } else {
            getHighlighterBar().css('left', (startDate.getTime()/viewDuration) * getEventWindowWidth() );
          }

          var width = ((startDate.getTime() - endDate.getTime())/viewDuration) * getEventWindowWidth();
          getHighlighterBar().css('width', width );
        }
        break;

      case 'duration':
        if( duration.length ) {

          switch( durationUnits ) {
            case 'day':
              eventDurationHours = duration * 24.0;
              break;
            case 'hour':
              eventDurationHours = duration;
              break;
            case 'minute':
              eventDurationHours = duration / 60.0;
              break;
            default:
              eventDurationHours = duration / 60.0;
          }

          var width = (eventDurationHours/getEventWindowHours()) * getEventWindowWidth();
          getHighlighterBar().css('width', width );

          if( isRTL ) {
            getHighlighterBar().css('left', getEventWindowWidth() - (((startDate.getTime() - viewStartTime)/viewDuration) * getEventWindowWidth()) - width );
          } else {
            getHighlighterBar().css('left', ((startDate.getTime() - viewStartTime)/viewDuration) * eventWindowW.width() );
          }
        }
        break;

      default:
        break;
    }

    //debugger;
  }

  getHighlighterBar().css('visibility', 'visible'); // TODO: Move this inside the 'if' statement
}
